#!/usr/bin/env node
// ---------------------------------------------------------------------------------------------------------------------
// This script is intended to generate type-definition for this module
// ---------------------------------------------------------------------------------------------------------------------

/* eslint-env node, es6 */
require('shelljs/global');

var path = require('path'),
    fs = require('fs'),
    chalk = require('chalk'),
    templates = require('./utils/templates'),

    IS_WINDOWS = (/^win/).test(process.platform),
    TARGET_DIR = path.join('types');

module.exports = function (exit) {
    console.log(chalk.yellow.bold('Generating type-definitions...'));

    try {
        // clean directory
        test('-d', TARGET_DIR) && rm('-rf', TARGET_DIR);
    }
    catch (e) {
        console.error(e.stack || e);
        return exit(e ? 1 : 0);
    }

    exec(`${IS_WINDOWS ? '' : 'node'} ${path.join('node_modules', '.bin', 'jsdoc')}${IS_WINDOWS ? '.cmd' : ''}` +
        ' -c .jsdoc-config-type-def.json -p', function (code) {

        if (!code) {
            fs.readFile(`${TARGET_DIR}/index.d.ts`, function (err, contents) {
                if (err) {
                    console.log(chalk.red.bold('unable to read the type-definition file'));
                    exit(1);
                }

                var source = contents.toString();
                source = source
                    // replacing Integer with number as 'Integer' is not a valid data-type in Typescript
                    .replace(/Integer/gm, 'number')
                    // replacing String[] with string[] as 'String' is not a valid data-type in Typescript
                    .replace(/String\[]/gm, 'string[]')
                    // replacing Boolean[] with boolean[] as 'Boolean' is not a valid data-type in Typescript
                    .replace(/Boolean\[]/gm, 'boolean[]')
                    // removing all occurrences html, as the these tags are not supported in Type-definitions
                    .replace(/<[^>]*>/gm, '')
                    // replacing @link tags with the object namepath to which it was linked,
                    // as these link tags are not navigable in type-definitions.
                    .replace(/\{@link (\w*)[#.]+(\w*)\}/gm, '$1.$2')
                    .replace(/\{@link (\S+)\}/gm, '$1'); // remove @link tags

                source = `${templates.heading}\n${source}`;

                fs.writeFile(`${TARGET_DIR}/index.d.ts`, source, function (err) {
                    if (err) {
                        console.log(chalk.red.bold('unable to write the type-definition file'));
                        exit(1);
                    }
                    console.log(chalk.green.bold(`type-definition file saved successfully at "${TARGET_DIR}"`));
                    exit(0);
                });
            });
        }
        else {
            // output status
            console.log(chalk.red.bold('unable to generate type-definition'));
            exit(code);
        }
    });
};

// ensure we run this script exports if this is a direct stdin.tty run
!module.parent && module.exports(exit);
